#include <ultra64.h>
#include "sm64.h"
#include "behavior_data.h"
#include "model_ids.h"
#include "seq_ids.h"
#include "dialog_ids.h"
#include "segment_symbols.h"
#include "level_commands.h"

#include "game/level_update.h"

#include "levels/scripts.h"

#include "actors/common1.h"
#include "actors/group14.h"

#include "make_const_nonconst.h"
#include "levels/bob/header.h"

const LevelScript level_bob_entry[] = {
	INIT_LEVEL(),
	LOAD_MIO0(0x07, _bob_segment_7SegmentRomStart, _bob_segment_7SegmentRomEnd), 
	LOAD_MIO0_TEXTURE(0x09, _generic_mio0SegmentRomStart, _generic_mio0SegmentRomEnd), 
	LOAD_MIO0(0x0A, _cloud_floor_skybox_mio0SegmentRomStart, _cloud_floor_skybox_mio0SegmentRomEnd), 
	LOAD_MIO0(0x05, _group3_mio0SegmentRomStart, _group3_mio0SegmentRomEnd), 
	LOAD_RAW(0x0C, _group3_geoSegmentRomStart, _group3_geoSegmentRomEnd), 
	LOAD_MIO0(0x06, _group14_mio0SegmentRomStart, _group14_mio0SegmentRomEnd), 
	LOAD_RAW(0x0D, _group14_geoSegmentRomStart, _group14_geoSegmentRomEnd), 
	LOAD_MIO0(0x08, _common0_mio0SegmentRomStart, _common0_mio0SegmentRomEnd), 
	LOAD_RAW(0x0F, _common0_geoSegmentRomStart, _common0_geoSegmentRomEnd), 
	ALLOC_LEVEL_POOL(),
	MARIO(MODEL_MARIO, 0x00000001, bhvMario), 
	JUMP_LINK(script_func_global_1), 
	JUMP_LINK(script_func_global_4), 
	JUMP_LINK(script_func_global_15), 
	LOAD_MODEL_FROM_GEO(MODEL_BOB_BUBBLY_TREE, bubbly_tree_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_BOB_CHAIN_CHOMP_GATE, bob_geo_000440), 
	LOAD_MODEL_FROM_GEO(MODEL_BOB_SEESAW_PLATFORM, bob_geo_000458), 
	LOAD_MODEL_FROM_GEO(MODEL_BOB_BARS_GRILLS, bob_geo_000470), 
	LOAD_MODEL_FROM_GEO(MODEL_TAILSDOLL, tailsdoll_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_NEWPIANO, newpiano_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_DISCOBOO, discoboo_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_BOOGIEBOO, boogieboo_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_BULLYCHAIR, bullychair_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_SCUTTLEBUGGO, scuttlebuggo_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_SWOOPBELL, swoopbell_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_BELLSIGN, bellsign_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_BULLYSONG, bullysong_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_WARP_PIPE, warp_pipe_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_WARP_PIPE2, warp_pipe2_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_WOODEN_DOOR, wooden_door_geo), 

	AREA(1, bob_area_1),
		WARP_NODE(0x0A, LEVEL_BOB, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF0, LEVEL_CASTLE_GROUNDS, 0x01, 0x0E, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF1, LEVEL_CASTLE_GROUNDS, 0x01, 0x0F, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0B, LEVEL_BOB, 0x01, 0x0C, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0C, LEVEL_BOB, 0x01, 0x0B, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0D, LEVEL_BOB, 0x02, 0x0A, WARP_NO_CHECKPOINT),
		OBJECT(MODEL_SWOOPBELL, 6824, -37, 4180, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT(MODEL_SWOOPBELL, 9964, -37, 4180, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT(MODEL_SWOOPBELL, 8725, 1898, 4180, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT(MODEL_SWOOPBELL, 9266, 3635, 3716, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT(MODEL_SWOOPBELL, 10710, 1898, 4180, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT(MODEL_SWOOPBELL, 8374, 939, 4180, 0, 0, 0, 0x00000000, bhvSwoop),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_008 << 16, bhvMessagePanel, ACT_1),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_170 << 16, bhvMessagePanel, ACT_2),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_171 << 16, bhvMessagePanel, ACT_3),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_172 << 16, bhvMessagePanel, ACT_4),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_173 << 16, bhvMessagePanel, ACT_5),
		OBJECT_WITH_ACTS(MODEL_BELLSIGN, -4151, -1439, 5533, 0, 0, 0, DIALOG_174 << 16, bhvMessagePanel, ACT_6),
		OBJECT(MODEL_DISCOBOO, 1170, -1783, -1111, 0, -63, 0, 0x00000000, bhvBoo),
		OBJECT(MODEL_DISCOBOO, -551, -1783, -967, 0, 142, 0, 0x00000000, bhvBoo),
		OBJECT(MODEL_DISCOBOO, 171, -1783, -1887, 0, 107, 0, 0x00000000, bhvBoo),
		OBJECT(MODEL_BOOGIEBOO, 440, -1587, -2571, 0, 0, 0, 0x03000000, bhvBalconyBigBoo),
		OBJECT(MODEL_CASTLE_WOODEN_DOOR, -27, -1884, -112, 0, 0, 0, 0x00000000, bhvDoor),
		OBJECT(MODEL_GOOMBA, 10773, 1087, 4034, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 7161, 2041, 4502, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 10300, 2972, 4150, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 8516, 2041, 3899, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 8228, 1087, 4337, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_NONE, 1739, 3853, 11782, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT_WITH_ACTS(MODEL_TAILSDOLL, -4871, -1854, -1017, 0, -42, 0, 0x05020000, bhvKoopaTD, ACT_6),
		OBJECT_WITH_ACTS(MODEL_NONE, 4489, -1020, -8896, 0, 0, 0, 0x00000000, bhvKoopaRaceEndpoint, ACT_6),
		OBJECT_WITH_ACTS(MODEL_NEWPIANO, -4626, -1854, -770, 0, -42, 0, 0x01000000, bhvTuxiesMother, ACT_2),
		OBJECT(MODEL_GOOMBA, 10336, -899, 4372, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 5512, -899, 3961, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_NONE, 8160, 2972, 4150, 0, 0, 0, 0x00000000, bhvGoombaTripletSpawner),
		OBJECT(MODEL_NONE, 10308, 3144, 5431, 0, 0, 0, 0x00750000, bhvPoleGrabbing),
		OBJECT(MODEL_NONE, 10664, 3294, 6598, 0, 0, 0, 0x00750000, bhvPoleGrabbing),
		OBJECT(MODEL_NONE, 10504, 3294, 8608, 0, 0, 0, 0x00750000, bhvPoleGrabbing),
		OBJECT(MODEL_NONE, 10997, 3394, 7770, 0, 0, 0, 0x00750000, bhvPoleGrabbing),
		OBJECT(MODEL_NONE, 10418, 3294, 9816, 0, 0, 0, 0x00750000, bhvPoleGrabbing),
		OBJECT(MODEL_BULLYSONG, 5379, 4565, 10854, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYSONG, 5654, 4565, 12655, 0, 150, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYSONG, 7013, 4278, 12331, 0, 150, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYSONG, 3590, 4774, 11726, 0, -180, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -6908, 8895, 871, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -7627, 8539, -1826, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -8446, 8456, -4165, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -10305, 8456, -4090, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -12400, 6617, -4924, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -6658, 8652, 5755, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -9896, 7480, 10435, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -5458, 9067, 2250, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -14595, 5053, 618, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -12759, 6617, -2917, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -12886, 7412, -8019, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -10154, 8539, -1826, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -6492, 8733, -238, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -7940, 8652, 8123, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -8080, 8275, 14443, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_BULLYCHAIR, -8957, 8057, 14882, 0, 0, 0, 0x00000000, bhvSmallBully),
		OBJECT(MODEL_CHAIN_CHOMP, 7390, -1857, -1739, 0, 0, 0, 0x00000000, bhvChainChomp),
		OBJECT(MODEL_CHAIN_CHOMP, 8989, -1857, -7307, 0, -45, 0, 0x00000000, bhvChainChomp),
		OBJECT(MODEL_GOOMBA, -6362, -1885, -854, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT_WITH_ACTS(MODEL_GOOMBA, -4924, -1885, -553, 0, 0, 0, 0x00000000, bhvGoomba, ACT_1 | ACT_3 | ACT_4 | ACT_5),
		OBJECT(MODEL_GOOMBA, -5027, -1885, 716, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, -6436, -1885, 675, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, -7338, -1888, -3202, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, -6310, -1573, -4213, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, -6524, -218, -6561, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 7788, 3970, 12392, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_GOOMBA, 9905, 3421, 11998, 0, 0, 0, 0x00000000, bhvGoomba),
		OBJECT(MODEL_PIRANHA_PLANT, 7413, -1857, -2797, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 3197, -1226, -11088, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 2520, -1024, -8295, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 9319, -1857, -882, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 7621, -1857, -6336, 0, 0, 0, 0x00000000, bhvPiranhaPlant),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 318, -1406, -12477, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 2320, -1263, -11063, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 2161, -1088, -8543, -20, -73, 25, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 5620, -1020, -8601, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 5821, -2054, -11173, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 2568, -2054, -13109, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -1786, -1400, -12168, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 8235, -1857, -5569, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 9005, -1857, -66, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, 2275, -1887, -1317, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -1717, -1887, -1162, 0, -74, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_NONE, -4249, -1276, 6849, 0, -180, 0, 0x000A0000, bhvSpinAirborneWarp),
		OBJECT_WITH_ACTS(MODEL_NEWPIANO, -6056, 1796, -10436, 0, 0, 0, 0x00000000, bhvSmallPenguin, ACT_2),
		OBJECT(MODEL_PIRANHA_PLANT, 600, -1400, -11584, 1, -32, 0, 0x00000000, bhvFirePiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 574, -1332, -9245, 28, -102, -21, 0x00000000, bhvFirePiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 4699, -1020, -10535, -20, -73, 25, 0x00000000, bhvFirePiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 4340, -2012, -11465, 0, 142, 0, 0x00000000, bhvFirePiranhaPlant),
		OBJECT(MODEL_PIRANHA_PLANT, 1603, -2005, -13628, 0, -10, 0, 0x00000000, bhvFirePiranhaPlant),
		OBJECT(MODEL_WOODEN_SIGNPOST, -14193, 4899, 2327, 0, 0, 0, DIALOG_015 << 16, bhvMessagePanel),
		OBJECT(MODEL_WOODEN_SIGNPOST, -6513, -218, -7145, 0, 0, 0, DIALOG_061 << 16, bhvMessagePanel),
		OBJECT(MODEL_WOODEN_SIGNPOST, -1001, -1887, -101, 0, 0, 0, DIALOG_060 << 16, bhvMessagePanel),
		OBJECT(MODEL_WOODEN_SIGNPOST, -4280, -76, 7420, 0, -180, 0, DIALOG_019 << 16, bhvMessagePanel),
		OBJECT(MODEL_WOODEN_SIGNPOST, 6911, 4278, 10639, 0, 0, 0, DIALOG_032 << 16, bhvMessagePanel),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -13370, 407, -7006, 0, 0, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -12023, -446, 436, 0, 0, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -7873, 1583, -8048, 0, 0, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -5139, 1583, -8048, 0, 0, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_BOB_BUBBLY_TREE, -11599, 153, -3214, 0, 0, 0, 0x00000000, bhvTree),
		OBJECT(MODEL_WARP_PIPE, -2027, -724, 3661, 0, -90, 0, 0x000B0000, bhvWarpPipe),
		OBJECT(MODEL_WARP_PIPE, -11560, 4861, 2341, 0, 0, 0, 0x000C0000, bhvWarpPipe),
		OBJECT(MODEL_WARP_PIPE2, 7566, -1287, -5031, 0, 0, 0, 0x000D0000, bhvWarpPipe),
		TERRAIN(bob_area_1_collision),
		MACRO_OBJECTS(bob_area_1_macro_objs),
		SET_BACKGROUND_MUSIC(0x00, SEQ_STREAMED_MUSICPLANT_ACT1),
		SHOW_DIALOG(0x00, DIALOG_018),
		TERRAIN_TYPE(TERRAIN_GRASS),
	END_AREA(),

	AREA(2, bob_area_2),
		WARP_NODE(0x0A, LEVEL_BOB, 0x01, 0x0D, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0E, LEVEL_BOB, 0x02, 0x0F, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0F, LEVEL_BOB, 0x02, 0x0E, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF0, LEVEL_CASTLE_GROUNDS, 0x01, 0x0E, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF1, LEVEL_BOB, 0x02, 0x1A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x1A, LEVEL_BOB, 0x02, 0x1A, WARP_NO_CHECKPOINT),
		OBJECT(MODEL_WOODEN_SIGNPOST, -564, 10769, -1334, 0, 0, 0, DIALOG_016 << 16, bhvMessagePanel),
		OBJECT(MODEL_WARP_PIPE2, 1117, 10769, -510, 0, 0, 0, 0x000A0000, bhvWarpPipe),
		OBJECT(MODEL_NONE, -358, 10769, 2, 0, 0, 0, 0x001A0000, bhvSpinAirborneWarp),
		OBJECT(MODEL_WARP_PIPE, -995, 10769, -510, 0, 0, 0, 0x000E0000, bhvWarpPipe),
		OBJECT(MODEL_WARP_PIPE, -3649, -8713, -957, 0, 90, 0, 0x000F0000, bhvWarpPipe),
		OBJECT(MODEL_NONE, -3633, -8344, -2480, 0, 0, 0, 0x04000000, bhvHiddenRedCoinStar),
		OBJECT(MODEL_NONE, -3633, -8645, -2480, 0, 0, 0, 0x00000000, bhvRedCoinStarMarker),
		TERRAIN(bob_area_2_collision),
		MACRO_OBJECTS(bob_area_2_macro_objs),
		SET_BACKGROUND_MUSIC(0x00, SEQ_STREAMED_MUSICPLANT_ACT2),
		TERRAIN_TYPE(TERRAIN_GRASS),
	END_AREA(),

	FREE_LEVEL_POOL(),
	MARIO_POS(1, 0, 0, 0, 0),
	CALL(0, lvl_init_or_update),
	CALL_LOOP(1, lvl_init_or_update),
	CLEAR_LEVEL(),
	SLEEP_BEFORE_EXIT(1),
	EXIT(),
};
