#include <ultra64.h>
#include "sm64.h"
#include "behavior_data.h"
#include "model_ids.h"
#include "seq_ids.h"
#include "dialog_ids.h"
#include "segment_symbols.h"
#include "level_commands.h"

#include "game/level_update.h"

#include "levels/scripts.h"

#include "actors/common1.h"
#include "actors/group10.h"

#include "make_const_nonconst.h"
#include "levels/castle_grounds/header.h"

const LevelScript level_castle_grounds_entry[] = {
	INIT_LEVEL(),
	LOAD_MIO0(0x07, _castle_grounds_segment_7SegmentRomStart, _castle_grounds_segment_7SegmentRomEnd), 
	LOAD_MIO0(0x0A, _water_skybox_mio0SegmentRomStart, _water_skybox_mio0SegmentRomEnd), 
	LOAD_MIO0_TEXTURE(0x09, _outside_mio0SegmentRomStart, _outside_mio0SegmentRomEnd), 
	LOAD_MIO0(0x05, _group10_mio0SegmentRomStart, _group10_mio0SegmentRomEnd), 
	LOAD_RAW(0x0C, _group10_geoSegmentRomStart, _group10_geoSegmentRomEnd), 
	LOAD_MIO0(0x06, _group15_mio0SegmentRomStart, _group15_mio0SegmentRomEnd), 
	LOAD_RAW(0x0D, _group15_geoSegmentRomStart, _group15_geoSegmentRomEnd), 
	LOAD_MIO0(0x08, _common0_mio0SegmentRomStart, _common0_mio0SegmentRomEnd), 
	LOAD_RAW(0x0F, _common0_geoSegmentRomStart, _common0_geoSegmentRomEnd), 
	ALLOC_LEVEL_POOL(),
	MARIO(MODEL_MARIO, 0x00000001, bhvMario), 
	JUMP_LINK(script_func_global_1), 
	JUMP_LINK(script_func_global_11), 
	JUMP_LINK(script_func_global_16), 
	LOAD_MODEL_FROM_GEO(MODEL_LEVEL_GEOMETRY_03, castle_grounds_geo_0006F4), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_BUBBLY_TREE, bubbly_tree_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_WARP_PIPE, warp_pipe_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_CASTLE_DOOR, castle_door_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_METAL_DOOR, metal_door_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_VCUTM_GRILL, castle_grounds_geo_00070C), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_FLAG, castle_grounds_geo_000660), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_GROUNDS_CANNON_GRILL, castle_grounds_geo_000724), 
	LOAD_MODEL_FROM_GEO(MODEL_VANILLAFIX, vanillafix_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CHAOUPSET, chaoupset_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_MARK, mark_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_GEMERL, gemerl_geo), 
	LOAD_MODEL_FROM_GEO(MODEL_CASTLE_DOOR_0_STARS, castle_door_0_star_geo), 

	AREA(1, castle_grounds_area_1),
		WARP_NODE(0x0A, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF0, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF1, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0B, LEVEL_BOB, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0E, LEVEL_CASTLE_GROUNDS, 0x01, 0x0E, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0F, LEVEL_CASTLE_GROUNDS, 0x01, 0x0F, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0C, LEVEL_BOWSER_1, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x1A, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x1F, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x1E, LEVEL_CASTLE_GROUNDS, 0x02, 0x0A, WARP_NO_CHECKPOINT),
		OBJECT(MODEL_CHAOUPSET, 412, -35, -726, 0, 55, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, 354, -51, -455, 0, 123, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, 710, -68, -547, 0, -59, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, -818, -88, 886, 0, 81, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, 1563, 159, 1286, 0, -116, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, 1960, 927, 1226, 0, -156, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, 2198, 651, -78, 0, -106, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_CHAOUPSET, -682, -47, -537, 0, 55, 0, 0x00000000, bhvChao),
		OBJECT(MODEL_GEMERL, 811, -34, -1462, 0, -54, 0, DIALOG_002 << 24, bhvToadMessageGemerl),
		OBJECT(MODEL_MARK, 2668, 855, 56, 0, -109, 0, 0x00000000, bhvMark),
		OBJECT(MODEL_NONE, 2656, 855, 52, 0, -109, 0, DIALOG_003 << 16, bhvSignOnWall),
		OBJECT(MODEL_WOODEN_SIGNPOST, 1759, 0, 637, 0, -109, 0, DIALOG_004 << 16, bhvMessagePanel),
		OBJECT(MODEL_CASTLE_DOOR_0_STARS, 1880, 0, 542, 0, -109, 0, 0x06000000, bhvDoor),
		OBJECT(MODEL_VANILLAFIX, 541, -51, -631, 0, 0, 0, DIALOG_001 << 24, bhvToadMessageVanilla),
		OBJECT(MODEL_NONE, -672, 218, -1659, -2, -158, 3, 0x000E0000, bhvPaintingStarCollectWarp),
		OBJECT(MODEL_NONE, -672, 218, -1659, -2, -158, 3, 0x000F0000, bhvPaintingDeathWarp),
		OBJECT(MODEL_NONE, -731, -29, 1635, 0, 151, 0, 0x000A0000, bhvSpinAirborneWarp),
		MARIO_POS(0x01, 151, -731, -29, 1635),
		OBJECT(MODEL_NONE, -601, 0, -1480, 0, 0, 0, 0x000B0000, bhvFadingWarp),
		OBJECT(MODEL_NONE, -918, -40, 1933, 0, -122, 0, 0x001E0000, bhvWarp),
		OBJECT(MODEL_NONE, 2117, 0, 617, 0, -109, 0, 0x000C0000, bhvWarp),
		OBJECT(MODEL_NONE, 1482, 0, 433, 0, -109, 0, 0x001A0000, bhvDeathWarp),
		OBJECT(MODEL_NONE, -68, -58, -860, 0, -89, 0, 0x001F0000, bhvAirborneWarp),
		TERRAIN(castle_grounds_area_1_collision),
		MACRO_OBJECTS(castle_grounds_area_1_macro_objs),
		SET_BACKGROUND_MUSIC(0x00, SEQ_STREAMED_CHAO_GARDEN),
		TERRAIN_TYPE(TERRAIN_GRASS),
	END_AREA(),

	AREA(2, castle_grounds_area_2),
		WARP_NODE(0x0A, LEVEL_CASTLE_GROUNDS, 0x02, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF0, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0xF1, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		WARP_NODE(0x0C, LEVEL_CASTLE_GROUNDS, 0x01, 0x0A, WARP_NO_CHECKPOINT),
		OBJECT(MODEL_CASTLE_DOOR_0_STARS, -327, 115, 118, 0, 75, 0, 0x00000000, bhvDoor),
		OBJECT(MODEL_WOODEN_SIGNPOST, 4011, 170, -431, 0, 90, 0, DIALOG_050 << 16, bhvMessagePanel),
		OBJECT(MODEL_NONE, 408, 170, 106, 0, 90, 0, DIALOG_051 << 16, bhvSignOnWall),
		OBJECT(MODEL_NONE, 4447, 170, -233, 0, 90, 0, 0x000A0000, bhvSpinAirborneWarp),
		OBJECT(MODEL_NONE, -597, 145, 61, 0, -15, 0, 0x000C0000, bhvWarp),
		TERRAIN(castle_grounds_area_2_collision),
		MACRO_OBJECTS(castle_grounds_area_2_macro_objs),
		SET_BACKGROUND_MUSIC(0x00, SEQ_STREAMED_VANILLAHOUSE),
		TERRAIN_TYPE(TERRAIN_GRASS),
	END_AREA(),

	FREE_LEVEL_POOL(),
	MARIO_POS(0x01, 151, -731, -29, 1635),
	CALL(0, lvl_init_or_update),
	CALL_LOOP(1, lvl_init_or_update),
	CLEAR_LEVEL(),
	SLEEP_BEFORE_EXIT(1),
	EXIT(),
};
